// Arduino I2C Bibliothek
#include <Wire.h>
// I2C Adresse des HMC5883
#define HMC_ADRESS 0b00011110

void setup() {
  // Initialisieren der Schnittstelle zum Host
  Serial.begin(115000);
  while (!Serial) {
  }
  Serial.println(F("Init I2C"));

  // I2C Bibliothek und SChnittstelle initialisieren
  Wire.begin();

  //Initialisieren des HMC5883
  Wire.beginTransmission(HMC_ADRESS);
  // Register 2 für den Modus
  Wire.write(0x02);
  // continuous measurement mode
  Wire.write(0x00);
  Wire.endTransmission();
}

char linedata[120];
int mx, my, mz;
#define MAG_MESSAGE PSTR("MAG: x=%i, y=%i, z=%i")

/** main loop. **/
void loop() {
  //Start der Leseoperation
  Wire.beginTransmission(HMC_ADRESS);
  // Erstes Register ist das MSB X Register (3)
  Wire.write(0x03); //select register 3, X MSB register
  Wire.endTransmission();

  //Lesen der Werte, da der HMC5883 automatisch das Register inkrementiert,
  // können alle 6 Register direkt nacheinander gelesen werden. Folge ist immer MSB,LSB und X,Z,Y
  Wire.requestFrom(HMC_ADRESS, 6);
  if (6 <= Wire.available()) {
    mx = Wire.read() << 8;
    mx |= Wire.read();
    mz = Wire.read() << 8;
    mz |= Wire.read();
    my = Wire.read() << 8;
    my |= Wire.read();
  }

  // Ausgabe der Werte
  sprintf_P(linedata, MAG_MESSAGE, mx, my, mz);
  Serial.println(linedata);

  // Berechnug des atasächlichen Winkels
  // Die berechnung ist nur korrekt, wenn der Sensor eben aufliegt. float dir = atan2(my , mx);
  if (dir < 0) {
    dir +=  2 * PI;
  }
  dir = dir * 180 / 3.1415;
  Serial.print("Richtung: ");
  if ((dir < 22.5) || (dir > 337.5 ))
    Serial.print("Nord");
  if ((dir > 22.5) && (dir < 67.5 ))
    Serial.print("Nord-Ost");
  if ((dir > 67.5) && (dir < 112.5 ))
    Serial.print("Ost");
  if ((dir > 112.5) && (dir < 157.5 ))
    Serial.print("Sued-Ost");
  if ((dir > 157.5) && (dir < 202.5 ))
    Serial.print("Sued");
  if ((dir > 202.5) && (dir < 247.5 ))
    Serial.print("Sued-West");
  if ((dir > 247.5) && (dir < 292.5 ))
    Serial.print("West");
  if ((dir > 292.5) && (dir < 337.5 ))
    Serial.print("Nord-West");

  Serial.print(", Grad:");
  Serial.println(dir, 2);
  delay(250);
}