#include <Wire.h>

// MCP4725A0 mit A0 auf Masse
#define MCP4725 0x60

// FastModus: schreibt nur ins DAC Register
#define CMD_FASTWRITEDAC        0x00
// schreibt nur ins DAC Register
#define CMD_WRITEDAC            0x40
// schreibt sowohl ins DAC wie auch ins EEPROM
#define CMD_WRITEDACEEPROM      0x60


void setup() {
  // Start mit I2C
  Wire.begin();
  // Geschwindigkeit 400KHz
  Wire.setClock(400000L);
}

void loop() {
  // einfacher Sägezahn
  for (int i = 0; i < 4096; i++) {
    setVoltage(i, false);
    // setFastVoltage(i);
    delay(20);
  }
}

void setVoltage(word voltage, bool writeEEPROM ) {
  Wire.beginTransmission(MCP4725);
  if (writeEEPROM) {
    Wire.write(CMD_WRITEDACEEPROM);
  } else {
    Wire.write(CMD_WRITEDAC);
  }
  // obere 8 bits schreiben
  Wire.write(voltage / 16);
  // untere 4 bits schreiben, dsa kleine Nibbel ist unbenutzt
  Wire.write((voltage % 16) << 4);
  Wire.endTransmission();
}

void setFastVoltage(word voltage) {
  Wire.beginTransmission(MCP4725);
  // Fast Modus, oberes Nibbel
  Wire.write(CMD_FASTWRITEDAC || ((voltage >> 8) && 0x0F));
  // unteres Byte
  Wire.write(voltage && 0x00FF);
  Wire.endTransmission();
}

