#include "FastSPI_LED2.h"
#include <DS3231.h>
#include <Wire.h>

// Anzahl der LEDs im Ring
#define NUM_LEDS 24

// Datenpin für die LED Ausgabe
#define DATA_PIN 6

// Farbfeld mit dem Farbwerb pro LED
CRGB leds[NUM_LEDS];

// Taster zum Stellen der Uhr
const byte SWITCH_HOUR = 3;
const byte SWITCH_MIN = 2;

DS3231 Clock;
bool PM = false;
bool h12 = true;
bool Century = false;

void setup() {
  Serial.begin(57600);
  pinMode(SWITCH_MIN, INPUT_PULLUP);
  pinMode(SWITCH_HOUR, INPUT_PULLUP);

  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS);

  Wire.begin();

  Clock.setClockMode(true);
}

const word WAIT_DELAY = 10;
CRGB color;
byte index = 0;
byte prevSensorValue = 0;
long myNullTime = 0;

void loop() {
  if (digitalRead(SWITCH_HOUR) == 0) {
    addHour();
  }
  if (digitalRead(SWITCH_MIN) == 0) {
    addMin();
  }

  FastLED.setBrightness(255);
  showTime();

  delay(500);
  ReadDS3231();
}

void addHour() {
  delay(100);
  while (digitalRead(SWITCH_HOUR) == 0) {
    delay(500);
    byte hour = Clock.getHour(h12, PM);
    hour++;
    if (hour > 23) {
      hour = 0;
    }
    Clock.setHour(hour);
    showTime();
  }
}

void addMin() {
  delay(100);
  while (digitalRead(SWITCH_MIN) == 0) {
    delay(100);
    byte mm = Clock.getMinute();
    mm++;
    if (mm > 60) {
      mm = 0;
    }
    Clock.setMinute(mm);
    showTime();
  }
}

void setLED(byte index, CRGB color) {
  Serial.print(index);
  Serial.print(",");
  index = index % NUM_LEDS;
  index = (NUM_LEDS) - index;
  index = index % NUM_LEDS;
  leds[index] = color;
  Serial.println(index);
}

void showTime() {
  // Markierungen setzen
  for (byte i = 0; i < NUM_LEDS; i++) {
    leds[i] = 0x020202;
    if ((i % 2) == 0) leds[i] = 0x101010;
    if ((i % 6) == 0) leds[i] = 0x404040;
  }

  int h = Clock.getHour(h12, PM);
  h = (h * NUM_LEDS / 12);

  int mm = Clock.getMinute();
  mm = mm * NUM_LEDS / 60;

  long sec = Clock.getSecond();
  if (sec == 0) {
    myNullTime = millis();
  }
  sec = (millis() - myNullTime) * NUM_LEDS / 60 / 1000;
  byte ss = sec % 256;
  setLED(ss, CRGB::Blue);

  setLED(mm, CRGB::Green);
  setLED(h, CRGB::Red);

  FastLED.show();
}

void ReadDS3231() {
  int second, minute, hour, date, month, year, temperature;
  second = Clock.getSecond();
  minute = Clock.getMinute();
  hour = Clock.getHour(h12, PM);
  date = Clock.getDate();
  month = Clock.getMonth(Century);
  year = Clock.getYear();

  temperature = Clock.getTemperature();

  Serial.print("20"); 
  Serial.print(year, DEC); Serial.print('-');
  Serial.print(month, DEC); Serial.print('-');
  Serial.print(date, DEC); Serial.print(' ');
  Serial.print(hour, DEC); Serial.print(':');
  Serial.print(minute, DEC); Serial.print(':'); Serial.print(second, DEC); 
  Serial.print("Temperature="); Serial.println(temperature);
}

