#include <Servo.h>
#include <EEPROM.h>
#include <LedControl.h>

/*
 Now we need a LedControl to work with.
 ***** These pin numbers will probably not work with your hardware *****
 pin 12 is connected to the DataIn 
 pin 11 is connected to the CLK 
 pin 10 is connected to LOAD 
 We have only a single MAX72XX.
 */
LedControl lc=LedControl(12,11,10,1);

#define MAX_KEYS  5
#define ANALOG_JITTER 50

const int keys [] = {0, 146, 330, 505, 741};

// elngth of the pin key
#define PIN_COUNT 4
// keyboard
#define PIN_KEYS A0
// Taster zum prüfen, ob die Tür zu ist
#define PIN_DOOR_CHECK 8 
// Notfalltaster
#define PIN_RESET 1
// Schloss (Servomotor) 
#define PIN_SERVO_DOOR 9
#define SERVO_DOOR_OPEN 0
#define SERVO_DOOR_CLOSE 180
#define PIN_ALARM 4

byte keypressed [PIN_COUNT] = { 255,255,255,255};
byte oldKey = 255;
byte pin[PIN_COUNT] = {255,255,255,255};

boolean doorOpen = false;
byte pinTrys;
Servo door;

void setup() {
  lc.shutdown(0,false);
  /* Set the brightness to a medium values */
  lc.setIntensity(0,15);
  /* and clear the display */
  lc.clearDisplay(0);

  // for reading keyboard
  analogReference(DEFAULT);

  pinMode(PIN_DOOR_CHECK, INPUT_PULLUP);
  pinMode(PIN_RESET, INPUT_PULLUP);
  pinMode(PIN_ALARM, OUTPUT);
  digitalWrite(PIN_ALARM, 0);

  door.attach(PIN_SERVO_DOOR);
  if (digitalRead(PIN_DOOR_CHECK) == 1) {
    // DOOR OPEN
    doorOpen = true;
    door.write(SERVO_DOOR_OPEN);
  } 
  else {
    // DOOR CLOSE
    doorOpen = false;
    door.write(SERVO_DOOR_CLOSE);
  }

  pinTrys = 3;

  if (digitalRead(PIN_RESET) == 0) {
    for (int i = 0; i < PIN_COUNT; i++) {
      pin[i] = 255;    
    }
    writePin();
    door.write(SERVO_DOOR_OPEN);
  }

  readPin();
}

void loop() {
  // Sensorwert lesen
  int sensorValue = analogRead(PIN_KEYS);    
  if (isNoPin(keypressed)) {
    lc.clearDisplay(0);
    lc.setDigit(0,0,0,true);
  }

  byte key = getKeys(sensorValue);
  if (key != oldKey) {
    if (key < 10) {
      for (int i = PIN_COUNT-1; i > 0; i--) {
        keypressed[i] = keypressed[i-1];
      }
      keypressed[0] = key;
      delay (250);
    } 
    else if (key == 11) {
      // reset
      emptyKeys();
    } 
    else if (key == 10) {
      if (doorOpen && isNoPin(keypressed) && isPinCorrect(pin)) {
        // Wenn die Tür offen ist und kein neuer Pin eingegeben wurde und der alte Pin auch nicht leer ist, 
        // dann Tür verschliessen
        doorOpen = false;
      } 
      else if (doorOpen && isPinCorrect(keypressed)) {
        // Wenn Tür offen und ein neuer Pin eingegeben wurde, dann Tür wieder zu und pin speichern
        doorOpen = false;

        for (byte i = 0; i < PIN_COUNT; i++) {
          pin[i] = keypressed[i];
        }
        writePin();
      }
      else if (!doorOpen) {
        // Tür ist zu
        if (testPin()) {
          // richtige Pin eingegeben, Tür öffnen
          doorOpen = true;
          pinTrys = 3;
          digitalWrite(PIN_ALARM, 0);
        } 
        else {
          lc.clearDisplay(0);
          pinTrys -= 1;
          if (pinTrys <= 0) {
            digitalWrite(PIN_ALARM, 1);
          }
        }  
      }
      emptyKeys();
    }
    oldKey = key; 
  }

  for (byte i = 0; i < PIN_COUNT; i++) {
    if (keypressed[i] < 100) {
      lc.setDigit(0,i,keypressed[i],false);
    }
  }

  doDoor();
  delay (100);
}

byte getKeys(int value) {
  byte x = 0;
  while (x < MAX_KEYS) {
    if ((value >= keys[x]-ANALOG_JITTER) && (value <= keys[x]+ANALOG_JITTER)) {
      return x;
    }
    x++;
  } 
  return 255;
}

boolean testPin() {
  boolean test = true;
  for (int i = 0; i < PIN_COUNT; i++) {
    test = test & (pin[i] == keypressed[i]);
  }
  return test;
}

void readPin() {
  for (int i = 0; i < PIN_COUNT; i++) {
    pin[i] = EEPROM.read(i);
    if (pin[i] > 9) {
      pin[i] = 255;
    }
  }
}

void writePin() {
  for (int i = 0; i < PIN_COUNT; i++) {
    if (pin[i] > 9) {
      pin[i] = 255;
    }
    EEPROM.write(i,pin[i]);
  }
}

boolean isPinCorrect(byte pinArray[] ) {
  boolean test = true;
  for (int i = 0; i < PIN_COUNT; i++) {
    test = test & (pinArray[i] <= 9);
  }
  return test;
}

// im Array sind alle Werte = 255 also unbesetzt
boolean isNoPin(byte pinArray[] ) {
  boolean test = true;
  for (int i = 0; i < PIN_COUNT; i++) {
    test = test & (pinArray[i] == 255);
  }
  return test;
}

void emptyKeys() {
  for (byte i = 0; i < PIN_COUNT; i++) {
    keypressed[i] = 255;
  }
}

// je nach Status wird hier die Tür verschossen oder geöffnet. 
void doDoor() {
  byte angle = doorOpen ? SERVO_DOOR_OPEN : SERVO_DOOR_CLOSE;
  if (door.read() != angle) {

    if (!doorOpen) {
      while (digitalRead(PIN_DOOR_CHECK) == 1) {
        // warten bis die Tür zugedrückt wurde
        delay(100);
      }
      // closing door
      lc.clearDisplay(0);
      lc.setChar(0,0,'E',false);         
      lc.setChar(0,1,'S',false);         
      lc.setChar(0,2,'O',false);         
      lc.setChar(0,3,'L',false);         
      lc.setChar(0,4,'C',false);

    }
    else {
      // opening door
      lc.clearDisplay(0);
      lc.setChar(0,0,'N',false);         
      lc.setChar(0,1,'E',false);         
      lc.setChar(0,2,'P',false);         
      lc.setChar(0,3,'O',false);         
    }

    door.write(angle);

    delay(250); // ein bischen warten bis der Servo soweit ist... 
  }
}




