/*
  DMX2RGBLED.ino
  Copyright (C) 2015 Wilfried Klaas <w.klaas@gmx.de>.  All right reserved.

  This sketch is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 3 of the License, or (at your option) any later version.

  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Conceptinetics.h>
#include "Adafruit_NeoPixel.h"

/* Um das Interface zu benutzen müssen folgende Jumper gesetzt sein.
 * RX-uart, TX-uart, Slave, und EN
 * Zum Hochladen des Sketches muss der EN Jumper auf Nicht EN gesetzt werden.
 *
 * Der DMX Slave wertet ab Kanal 1 (DMX_SLAVE_STARTADRESS) 10 Kanäle (DMX_SLAVE_CHANNELS) aus
 * und gibt diese dann an die RGB LED Kette vom Typ WS2812B weiter.
 * Kanal 1 ist dabei rot, 2 grün, 3 blau, 4 bestimmt die Verzögerungszeit, mit der die Farben durchgeleitet werden.
 * Ist der Zeitwert < 10 werden alle LEDs gleichzeitig verändert.
 */

// Konstanten für die DMX Startadresse und die Anzahl der Kanäle die der DMX Empfänger auswerten soll.
#define DMX_SLAVE_STARTADRESS  1
#define DMX_SLAVE_CHANNELS    10

// Erzeugen eines DMX_Slave Kontrollers
DMX_Slave dmx_slave ( DMX_SLAVE_CHANNELS );

// Anzahl der RGB LEDs in der WS2812B Kette
#define NUM_LEDS 24

// Datenpin der RGB LEDs
#define DATA_PIN 11

// Initialisieren der Neopixel RGB LED Struktur
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUM_LEDS, DATA_PIN, NEO_GRB + NEO_KHZ800);

void setup() {
  // NeoPixel Bibliothek starten
  pixels.begin();
  pixels.setBrightness(255);
  
  // DMX Empfänger starten und Startadresse setzen
  dmx_slave.enable ();
  dmx_slave.setStartAddress (DMX_SLAVE_STARTADRESS);
}

void loop() {
  // Auslesen der Farben und der Verzögerungszeit
  byte red = dmx_slave.getChannelValue (1);
  byte green = dmx_slave.getChannelValue (2);
  byte blue = dmx_slave.getChannelValue (3);
  int delayTime = dmx_slave.getChannelValue (4);

  // und auf den Pixelring übertragen
  showColor(pixels.Color(red, green, blue), delayTime < 10);

  // Wartezeit einstellen, sollte min. 10 ms betragen, da es sonst zu aussetzern kommt.
  delay(delayTime > 10 ? delayTime : 10);
}

// Farbe für die gewünschten LED setzen
void showColor(uint32_t color, bool all) {
  if (all) {
    // Farbe für alle LEDs setzen
    for (int i = 0; i < NUM_LEDS; i++) {
      pixels.setPixelColor(i, color);
    }
  } else {
    // Farbe für die LEDs nachziehen
    for (int i = NUM_LEDS - 1; i > 0 ; i--) {
      pixels.setPixelColor(i, pixels.getPixelColor(i - 1));
    }
    // und aktuelle Farbe auf die 1. LED setzen
    pixels.setPixelColor(0, color);
  }
  // jetzt anzeigen
  pixels.show();
}

// alle Farben löschen
void clear() {
  for (int i = 0; i < NUM_LEDS; i++) {
    pixels.setPixelColor(i, pixels.Color(0, 0, 0));
  }
  pixels.show();
}
