/*
  DmxFadeUp.ino
  Copyright (C) 2015 Wilfried Klaas <w.klaas@gmx.de>.  All right reserved.

  This sketch is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 3 of the License, or (at your option) any later version.

  This sketch is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
 * DMX Fade Up Beispiel.
 * Es wird das DMX Interface von Conceptinetics benutzt. Als DMX Slave verwende ich 
 * hier einen Bühnenscheinwerfer mit einem einfachen 3 Kanal Interface. Jeder Kanal steuert direkt eine Farbe.
 * Da der Schweinwerfer ab Kanal 5 arbeitet, ist die Zuordnung folgende 
 * Kanal 5 = ROT, Kanal 6 = Grün, Kanal 7 = Blau
 */

/* Um das Interface zu benutzen müssen folgende Jumper gesetzt sein.
 * RX-uart, TX-uart, DE, und EN
 * Zum Hochladen des Sketches muss der EN Jumper auf Nicht EN gesetzt werden.
 */
#include <Conceptinetics.h>

// Kanäle des RGB Scheinwerfers
#define RED_CHANNEL 5
#define GREEN_CHANNEL 6
#define BLUE_CHANNEL 7

// Hier die maximale Anzahl der DMX Kanäle definieren
#define DMX_MASTER_CHANNELS 128 

// Pin zum Auswahl des READ/WRITE Modus auf dem Shield.
#define RXEN_PIN 2

// Erzeugen des DMX Master Kontrollers mit den vorher eingestellten Werten
// Der Master wird die DE Leitung zur Steuerung des READ/WRITE Modus benutzen
DMX_Master dmx_master ( DMX_MASTER_CHANNELS, RXEN_PIN );

void setup() {             
  // DMX Kontroller starten und mit der Sendung der DMX Befehle beginnen
  dmx_master.enable ();  
  // Kompletten DMX Kanalbereich löschen
  dmx_master.setChannelRange ( 1, DMX_MASTER_CHANNELS, 0 );
}

byte dimmer;
const byte MAX = 64;
byte red = MAX, green = 0, blue = 0;

void loop() {
  if (red == MAX) {
    red = MAX - 1;
    green = 1;
    blue = 0;
  } else if (red > 0 && green < MAX && blue == 0) {
    red--;
    green++;
  } else if (green == MAX) {
    green = MAX - 1;
    blue = 1;
    red = 0;
  } else if (green > 0 && blue < MAX && red == 0) {
    green--;
    blue++;
  } else if (blue == MAX) {
    blue = MAX - 1;
    red = 1;
    green = 0;
  } else if (blue > 0 && red < MAX && green == 0) {
    blue--;
    red++;
  }

  // Ausgeben der Farbwerte
  dmx_master.setChannelValue ( RED_CHANNEL, red << 2 );  
  dmx_master.setChannelValue ( GREEN_CHANNEL, green << 2);  
  dmx_master.setChannelValue ( BLUE_CHANNEL, blue << 2 );  
 
   // Kurze Pause gegen Flackern
  delay ( 10 );
}
