#include <debug.h>
#include <makros.h>

// Ausgang für die Motorkontrolle
#define MotorUp 8
#define MotorDown 9

#define ShutterUp 1
#define ShutterDown 0

// Einschaltzeit für den Motor
#define OnTime 10 * 1000

// digitale Eingänge für die Taster
#define SwitchUp 6
#define SwitchDown 7

// analoge Eingänge der Fotowiderstände
#define DawnSensor 0
#define SunSensor 1

// Zeitintervall für die Sonnensteuerung
#define WaitTime 20 * 1000

enum STATE {WAIT, GO_UP, GO_DOWN, OPEN, CLOSE, HALF};

STATE state, nextState;

void setup() {

  // Initialisierung der Ein/Ausgänge
  pinMode(MotorUp, OUTPUT);
  pinMode(MotorDown, OUTPUT);
  pinMode(SwitchUp, INPUT_PULLUP);
  pinMode(SwitchDown, INPUT_PULLUP);

  Serial.begin(9600);

  if (!Serial) {
    // warten auf den Host (Leonardo)
  }

  state = OPEN;
  Serial.println("shutter v0.1");
}

long waitTime, startTime;
long tempTime;

void loop() {
  // Motor ausschalten
  checkMotorOff();

  // Taster für hoch gedrückt?
  doSwitchUp();
  // Taster für herrunter gedrückt?
  doSwitchDown();

  // Hostkommando erhalten?
  checkHost();

  // muss gewartet werden
  if (state == WAIT) {
    if ((millis() - waitTime) > WaitTime) {
      // Wartezeit abgelaufen, nächster Status
      state == nextState;
    }
  }
}

// Motor nach Ablaufzeit abschalten
void checkMotorOff() {
  if ((state == GO_UP) || (state == GO_DOWN)) {
    if (millis() - startTime > OnTime) {
      MotorOff();
      state = nextState;
      showInfo();
    }
  }
}

// Taster hoch gedrückt, dann Rolllade hoch
void doSwitchUp() {
  if (digitalRead(SwitchUp) == 0) {
    openShutter();
    showInfo();
  }
}

// Taster herrunter gedrückt, dann Rolllade runter
void doSwitchDown() {
  if (digitalRead(SwitchDown) == 0) {
    closeShutter();
    showInfo();
  }
}

// Hostkommando auswerten
void checkHost() {
  if (Serial.available() > 0) {
    byte myChar = Serial.read();
    if (myChar == 's') {
      showInfo();
    }
    if (myChar == 'r') {
      readSensors();
    }
    if (myChar == 'o') {
      openShutter();
      showInfo();
    }
    if (myChar == 'c') {
      closeShutter();
      showInfo();
    }
    if (myChar == 'h') {
      closeShutterHalf();
      showInfo();
    }
  }
}

// Lichtsensoren auswerten
void readSensors() {
  int dawnValue = analogRead(DawnSensor);
  Serial.print("d:");
  Serial.println(dawnValue, HEX);
  int sunValue = analogRead(SunSensor);
  Serial.print("w:");
  Serial.println(sunValue, HEX);
}

// aktuelle Status auf der Schnittstelle ausgeben
void showInfo() {
  Serial.print("s:");
  switch (state) {
    case OPEN:
      Serial.print("o");
      break;
    case WAIT:
      Serial.print("w");
      break;
    case GO_UP:
      Serial.print("u");
      break;
    case GO_DOWN:
      Serial.print("d");
      break;
    case CLOSE:
      Serial.print("c");
      break;
    case HALF:
      Serial.print("h");
      break;
    default:
      Serial.print("n");
  }
  Serial.println();
}

// Motor aus
void MotorOff() {
  digitalWrite(MotorUp, 0);
  digitalWrite(MotorDown, 0);
}

// Rolllade öffnen
void openShutter() {
  if ((state != OPEN) && (state != GO_UP)) {
    state = GO_UP;
    nextState = OPEN;
    startTime = millis();
    digitalWrite(MotorUp, 1);
  }
}

// Rolllade schliessen
void closeShutter() {
  if ((state != CLOSE) && (state != GO_DOWN)) {
    state = GO_DOWN;
    nextState = CLOSE;
    startTime = millis();
    digitalWrite(MotorDown, 1);
  }
}

// Rolllade auf Höhe Sonnensensor
void closeShutterHalf() {
  if (state == OPEN) {
    int sunLevel = analogRead(SunSensor);
    closeShutter();
    // close shutter while the sun sensor is in bright light
    while (between(sunLevel, sunLevel + 10, sunLevel - 10) )  {
      sunLevel = analogRead(SunSensor);
    }
    MotorOff();
    waitTime = millis();
    state = WAIT;
    nextState = HALF;
  }
}
